@@ -1,7 +1,7 @@
 #ifndef STAN_MATH_PRIM_SCAL_META_MAX_SIZE_MVT_HPP
 #define STAN_MATH_PRIM_SCAL_META_MAX_SIZE_MVT_HPP
 
-#include <stan/math/prim/scal/meta/length_mvt.hpp>
+#include "length_mvt.hpp"
 #include <cstdlib>
 
 namespace stan {
